/*
 * This class is responsible for generating all of the code for the ObjectGraph
 * class in the explanatory tool
 */

package simse.codegenerator.explanatorytoolgenerator;

import simse.codegenerator.CodeGeneratorConstants;
import simse.codegenerator.CodeGeneratorUtils;
import simse.modelbuilder.ModelOptions;
import simse.modelbuilder.objectbuilder.Attribute;
import simse.modelbuilder.objectbuilder.AttributeTypes;
import simse.modelbuilder.objectbuilder.DefinedObjectTypes;
import simse.modelbuilder.objectbuilder.NumericalAttribute;
import simse.modelbuilder.objectbuilder.SimSEObjectType;
import simse.modelbuilder.objectbuilder.SimSEObjectTypeTypes;
import simse.modelbuilder.startstatebuilder.CreatedObjects;
import simse.modelbuilder.startstatebuilder.SimSEObject;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Vector;

import javax.swing.JOptionPane;

public class ObjectGraphGenerator implements CodeGeneratorConstants {
  private File directory; // directory to save generated code into
  private DefinedObjectTypes objTypes;
  private CreatedObjects objects;
  private ModelOptions options;

  public ObjectGraphGenerator(DefinedObjectTypes objTypes, CreatedObjects objs,
      File directory, ModelOptions options) {
    this.objTypes = objTypes;
    this.objects = objs;
    this.directory = directory;
    this.options = options;
  }

  public void generate() {
    File objGraphFile = new File(directory,
        ("simse\\explanatorytool\\ObjectGraph.java"));
    if (objGraphFile.exists()) {
      objGraphFile.delete(); // delete old version of file
    }
    try {
      FileWriter writer = new FileWriter(objGraphFile);
      writer
          .write("/* File generated by: simse.codegenerator.explanatorytoolgenerator.ObjectGraphGenerator */");
      writer.write(NEWLINE);
      writer.write("package simse.explanatorytool;");
      writer.write(NEWLINE);
      writer.write(NEWLINE);
      writer.write("import simse.adts.objects.*;");
      writer.write(NEWLINE);
      writer.write("import simse.state.*;");
      writer.write(NEWLINE);
      writer.write("import simse.state.logger.Logger;");
      writer.write(NEWLINE);
      writer.write("import simse.SimSE;");
      writer.write(NEWLINE);
      writer.write(NEWLINE);
      writer.write("import org.jfree.chart.ChartFactory;");
      writer.write(NEWLINE);
      writer.write("import org.jfree.chart.ChartPanel;");
      writer.write(NEWLINE);
      writer.write("import org.jfree.chart.ChartRenderingInfo;");
      writer.write(NEWLINE);
      writer.write("import org.jfree.chart.JFreeChart;");
      writer.write(NEWLINE);
      writer.write("import org.jfree.chart.axis.NumberAxis;");
      writer.write(NEWLINE);
      writer.write("import org.jfree.chart.plot.PlotOrientation;");
      writer.write(NEWLINE);
      writer.write("import org.jfree.chart.plot.XYPlot;");
      writer.write(NEWLINE);
      writer
          .write("import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;");
      writer.write(NEWLINE);
      writer.write("import org.jfree.data.Range;");
      writer.write(NEWLINE);
      writer.write("import org.jfree.data.xy.XYDataset;");
      writer.write(NEWLINE);
      writer.write("import org.jfree.data.xy.XYSeries;");
      writer.write(NEWLINE);
      writer.write("import org.jfree.data.xy.XYSeriesCollection;");
      writer.write(NEWLINE);
      writer.write("import org.jfree.ui.RectangleEdge;");
      writer.write(NEWLINE);
      writer.write("import org.jfree.ui.RectangleInsets;");
      writer.write(NEWLINE);
      writer.write("import org.jfree.ui.RefineryUtilities;");
      writer.write(NEWLINE);
      writer.write(NEWLINE);
      writer.write("import java.awt.Color;");
      writer.write(NEWLINE);
      writer.write("import java.awt.event.ActionEvent;");
      writer.write(NEWLINE);
      writer.write("import java.awt.event.ActionListener;");
      writer.write(NEWLINE);
      writer.write("import java.awt.event.MouseEvent;");
      writer.write(NEWLINE);
      writer.write("import java.awt.event.MouseListener;");
      writer.write(NEWLINE);
      writer.write("import java.awt.geom.Point2D;");
      writer.write(NEWLINE);
      writer.write("import java.awt.geom.Rectangle2D;");
      writer.write(NEWLINE);
      writer.write("import java.awt.Point;");
      writer.write(NEWLINE);
      writer.write("import java.awt.event.WindowAdapter;");
      writer.write(NEWLINE);
      writer.write("import java.awt.event.WindowEvent;");
      writer.write(NEWLINE);
      writer.write("import java.util.ArrayList;");
      writer.write(NEWLINE);
      writer.write(NEWLINE);
      writer.write("import javax.swing.JFrame;");
      writer.write(NEWLINE);
      writer.write("import javax.swing.JMenuItem;");
      writer.write(NEWLINE);
      writer.write("import javax.swing.JOptionPane;");
      writer.write(NEWLINE);
      writer.write("import javax.swing.JSeparator;");
      writer.write(NEWLINE);
      writer.write(NEWLINE);
      writer.write("public class ObjectGraph extends JFrame implements MouseListener, ActionListener {");
      writer.write(NEWLINE);

      // member variables:
      writer.write("private ArrayList<State> log;");
      writer.write(NEWLINE);
      writer.write("private String objTypeType;");
      writer.write(NEWLINE);
      writer.write("private String objType;");
      writer.write(NEWLINE);
      writer.write("private String keyAttVal;");
      writer.write(NEWLINE);
      writer.write("private String[] attributes;");
      writer.write(NEWLINE);
      writer.write("private JFreeChart chart; // chart object");
      writer.write(NEWLINE);
    	writer.write("private ChartPanel chartPanel;");
    	writer.write(NEWLINE);
    	writer.write("private JMenuItem newBranchItem;");
    	writer.write(NEWLINE);
    	writer.write("private JSeparator separator;");
    	writer.write(NEWLINE);
    	writer.write("private int lastRightClickedX; // last x-val that was right-clicked on");
    	writer.write(NEWLINE);
    	writer.write("private XYSeries[] series;");
    	writer.write(NEWLINE);
    	writer.write("private Branch branch; // branch from which this graph is generated");
    	writer.write(NEWLINE);
      writer.write(NEWLINE);

      // constructor:
      writer
          .write("public ObjectGraph(String title, ArrayList<State> log, String objTypeType, String objType, String keyAttVal, String[] attributes, boolean showChart, Branch branch) {");
      writer.write(NEWLINE);
  		writer.write("super();");
  		writer.write(NEWLINE);
  		writer.write("this.branch = branch;");
  		writer.write(NEWLINE);
  		writer.write("if (branch.getName() != null) {");
  		writer.write(NEWLINE);
  		writer.write("title = title.concat(\" - \" + branch.getName());");
  		writer.write(NEWLINE);
  		writer.write(CLOSED_BRACK);
  		writer.write(NEWLINE);
  		writer.write("setTitle(title);");
      writer.write(NEWLINE);
      writer.write("this.log = log;");
      writer.write(NEWLINE);
      writer.write("this.objTypeType = objTypeType;");
      writer.write(NEWLINE);
      writer.write("this.objType = objType;");
      writer.write(NEWLINE);
      writer.write("this.keyAttVal = keyAttVal;");
      writer.write(NEWLINE);
      writer.write("this.attributes = attributes;");
      writer.write(NEWLINE);
      writer.write("lastRightClickedX = 0;");
      writer.write(NEWLINE);
      writer.write("XYDataset dataset = createDataset();");
      writer.write(NEWLINE);
      writer.write("chart = createChart(dataset);");
      writer.write(NEWLINE);
      writer.write("chartPanel = new ChartPanel(chart);");
      writer.write(NEWLINE);
      writer.write("chartPanel.addMouseListener(this);");
      writer.write(NEWLINE);
      writer
          .write("chartPanel.setPreferredSize(new java.awt.Dimension(500, 270));");
      writer.write(NEWLINE);
      writer.write("setContentPane(chartPanel);");
      writer.write(NEWLINE);
  		writer.write("newBranchItem = new JMenuItem(\"Start new game from here\");");
  		writer.write(NEWLINE);
  		writer.write("newBranchItem.addActionListener(this);");
  		writer.write(NEWLINE);
  		writer.write("separator = new JSeparator();");
  		writer.write(NEWLINE);
      writer.write("pack();");
      writer.write(NEWLINE);
      writer.write("RefineryUtilities.centerFrameOnScreen(this);");
      writer.write(NEWLINE);
      writer.write("setVisible(showChart);");
      writer.write(NEWLINE);
      writer.write(CLOSED_BRACK);
      writer.write(NEWLINE);
      writer.write(NEWLINE);

      // "createDataset" method:
      writer.write("// Creates the dataset for this graph");
      writer.write(NEWLINE);
      writer.write("private XYDataset createDataset() {");
      writer.write(NEWLINE);
      writer.write("series = new XYSeries[attributes.length];");
      writer.write(NEWLINE);
      writer.write("for (int i = 0; i < attributes.length; i++) {");
      writer.write(NEWLINE);
      writer.write("series[i] = new XYSeries(attributes[i]);");
      writer.write(NEWLINE);
      writer.write(CLOSED_BRACK);
      writer.write(NEWLINE);
      writer.write(NEWLINE);
      writer.write("for (int i = 0; i < log.size(); i++) {");
      writer.write(NEWLINE);
      writer.write("for (int j = 0; j < attributes.length; j++) {");
      writer.write(NEWLINE);

      // go through all object types and generate code for them:
      Vector<SimSEObjectType> types = objTypes.getAllObjectTypes();
      for (int i = 0; i < types.size(); i++) {
        SimSEObjectType type = types.get(i);
        String uCaseName = 
        	CodeGeneratorUtils.getUpperCaseLeading(type.getName());
        String lCaseName = type.getName().toLowerCase();
        if (i > 0) {
          writer.write("else ");
        }
        writer.write("if (objTypeType.equals(\""
            + SimSEObjectTypeTypes.getText(type.getType())
            + "\") && objType.equals(\"" + uCaseName + "\")) {");
        writer.write(NEWLINE);
        writer.write(uCaseName + " " + lCaseName + " = null;");
        writer.write(NEWLINE);

        // go through each created object of that type and generate code for it:
        Vector<SimSEObject> objsOfType = objects.getAllObjectsOfType(type);
        for (int j = 0; j < objsOfType.size(); j++) {
          SimSEObject obj = objsOfType.get(j);
          if (j > 0) {
            writer.write("else ");
          }
          writer.write("if (keyAttVal.equals(\""
              + obj.getKey().getValue().toString() + "\")) {");
          writer.write(NEWLINE);
          writer.write(lCaseName + " = log.get(i).get"
              + SimSEObjectTypeTypes.getText(type.getType())
              + "StateRepository().get" + uCaseName + "StateRepository().get(");
          if (obj.getKey().getAttribute().getType() == AttributeTypes.STRING) { 
            writer.write("\"" + obj.getKey().getValue().toString() + "\");");
          } else { // non-String attribute
            writer.write(obj.getKey().getValue().toString() + ");");
          }
          writer.write(NEWLINE);
          writer.write(CLOSED_BRACK);
          writer.write(NEWLINE);
        }
        writer.write("if (" + lCaseName + " != null) {");
        writer.write(NEWLINE);

        // go through each attribute for this type and generate code for it:
        Vector<Attribute> atts = type.getAllAttributes();
        boolean writeElse = false;
        for (Attribute att : atts) {
          if ((att instanceof NumericalAttribute)
              && (att.isVisible() || att.isVisibleOnCompletion())) {
            if (writeElse) {
              writer.write("else ");
            }
            writer.write("if (attributes[j].equals(\""
                + CodeGeneratorUtils.getUpperCaseLeading(att.getName()) + 
                "\")) {");
            writer.write(NEWLINE);
            writer.write("series[j].add(i, " + lCaseName + ".get"
                + CodeGeneratorUtils.getUpperCaseLeading(att.getName()) + 
                "());");
            writer.write(NEWLINE);
            writer.write(CLOSED_BRACK);
            writer.write(NEWLINE);
            writeElse = true;
          }
        }
        writer.write(CLOSED_BRACK);
        writer.write(NEWLINE);
        writer.write(CLOSED_BRACK);
        writer.write(NEWLINE);
      }
      writer.write(CLOSED_BRACK);
      writer.write(NEWLINE);
      writer.write(CLOSED_BRACK);
      writer.write(NEWLINE);
      writer.write("XYSeriesCollection dataset = new XYSeriesCollection();");
      writer.write(NEWLINE);
      writer.write("for (int i = 0; i < series.length; i++) {");
      writer.write(NEWLINE);
      writer.write("dataset.addSeries(series[i]);");
      writer.write(NEWLINE);
      writer.write(CLOSED_BRACK);
      writer.write(NEWLINE);
      writer.write("return dataset;");
      writer.write(NEWLINE);
      writer.write(CLOSED_BRACK);
      writer.write(NEWLINE);
      writer.write(NEWLINE);

      // "createChart" method:
      writer.write("// Creates the chart for this graph");
      writer.write(NEWLINE);
      writer.write("private JFreeChart createChart(XYDataset dataset) {");
      writer.write(NEWLINE);
      writer.write("// create the chart:");
      writer.write(NEWLINE);
      writer
          .write("JFreeChart chart = ChartFactory.createXYLineChart(this.getTitle(), \"Clock Ticks\", null, dataset, PlotOrientation.VERTICAL, true, true, false);");
      writer.write(NEWLINE);
      writer.write("chart.setBackgroundPaint(Color.white);");
      writer.write(NEWLINE);
      writer.write("XYPlot plot = (XYPlot) chart.getPlot();");
      writer.write(NEWLINE);
      writer.write("plot.setBackgroundPaint(Color.lightGray);");
      writer.write(NEWLINE);
      writer
          .write("plot.setAxisOffset(new RectangleInsets(5.0, 5.0, 5.0, 5.0));");
      writer.write(NEWLINE);
      writer.write("plot.setDomainGridlinePaint(Color.white);");
      writer.write(NEWLINE);
      writer.write("plot.setRangeGridlinePaint(Color.white);");
      writer.write(NEWLINE);
      writer
          .write("XYLineAndShapeRenderer renderer = (XYLineAndShapeRenderer) plot.getRenderer();");
      writer.write(NEWLINE);
      writer.write("renderer.setShapesVisible(true);");
      writer.write(NEWLINE);
      writer.write("renderer.setShapesFilled(true);");
      writer.write(NEWLINE);
      writer.write(NEWLINE);
      writer
          .write("// change the auto tick unit selection to integer units only:");
      writer.write(NEWLINE);
      writer
          .write("NumberAxis domainAxis = (NumberAxis) plot.getDomainAxis();");
      writer.write(NEWLINE);
      writer
          .write("domainAxis.setStandardTickUnits(NumberAxis.createIntegerTickUnits());");
      writer.write(NEWLINE);
      writer.write(NEWLINE);
      writer.write("return chart;");
      writer.write(NEWLINE);
      writer.write(CLOSED_BRACK);
      writer.write(NEWLINE);
      writer.write(NEWLINE);
      
      // "update" method:
    	writer.write("public void update() {");
    	writer.write(NEWLINE);
    	writer.write("if ((log.size() > 0) && (log.get(log.size() - 1) != null)) { // there is a log that has not been graphed yet");
    	writer.write(NEWLINE);
 			writer.write("// add a new end data point for each series/attribute");
 			writer.write(NEWLINE);
    	writer.write("for (int j = 0; j < attributes.length; j++) {");
    	writer.write(NEWLINE);
      // go through all object types and generate code for them:
      for (int i = 0; i < types.size(); i++) {
        SimSEObjectType type = types.get(i);
        String uCaseName = 
        	CodeGeneratorUtils.getUpperCaseLeading(type.getName());
        String lCaseName = type.getName().toLowerCase();
        if (i > 0) {
          writer.write("else ");
        }
        writer.write("if (objTypeType.equals(\""
            + SimSEObjectTypeTypes.getText(type.getType())
            + "\") && objType.equals(\"" + uCaseName + "\")) {");
        writer.write(NEWLINE);
        writer.write(uCaseName + " " + lCaseName + " = null;");
        writer.write(NEWLINE);

        // go through each created object of that type and generate code for it:
        Vector<SimSEObject> objsOfType = objects.getAllObjectsOfType(type);
        for (int j = 0; j < objsOfType.size(); j++) {
          SimSEObject obj = objsOfType.get(j);
          if (j > 0) {
            writer.write("else ");
          }
          writer.write("if (keyAttVal.equals(\""
              + obj.getKey().getValue().toString() + "\")) {");
          writer.write(NEWLINE);
          writer.write(lCaseName + " = log.get(log.size() - 1).get"
              + SimSEObjectTypeTypes.getText(type.getType())
              + "StateRepository().get" + uCaseName + "StateRepository().get(");
          if (obj.getKey().getAttribute().getType() == AttributeTypes.STRING) {
            writer.write("\"" + obj.getKey().getValue().toString() + "\");");
          } else { // non-String attribute
            writer.write(obj.getKey().getValue().toString() + ");");
          }
          writer.write(NEWLINE);
          writer.write(CLOSED_BRACK);
          writer.write(NEWLINE);
        }
        writer.write("if (" + lCaseName + " != null) {");
        writer.write(NEWLINE);

        // go through each attribute for this type and generate code for it:
        Vector<Attribute> atts = type.getAllAttributes();
        boolean writeElse = false;
        for (Attribute att : atts) {
          if ((att instanceof NumericalAttribute)
              && (att.isVisible() || att.isVisibleOnCompletion())) {
            if (writeElse) {
              writer.write("else ");
            }
            writer.write("if (attributes[j].equals(\""
                + CodeGeneratorUtils.getUpperCaseLeading(att.getName()) + 
                "\")) {");
            writer.write(NEWLINE);
            writer.write("series[j].add(log.size(), " + lCaseName + ".get"
                + CodeGeneratorUtils.getUpperCaseLeading(att.getName()) + 
                "());");
            writer.write(NEWLINE);
            writer.write(CLOSED_BRACK);
            writer.write(NEWLINE);
            writeElse = true;
          }
        }
        writer.write(CLOSED_BRACK);
        writer.write(NEWLINE);
        writer.write(CLOSED_BRACK);
        writer.write(NEWLINE);
      }
      writer.write(CLOSED_BRACK);
      writer.write(NEWLINE);
      writer.write(CLOSED_BRACK);
      writer.write(NEWLINE);
      writer.write(CLOSED_BRACK);
      writer.write(NEWLINE);
      writer.write(NEWLINE);

      // mouseReleased function:
    	writer.write("// responds to RIGHT-clicks on the chart");
    	writer.write(NEWLINE);
    	writer.write("public void mouseReleased(MouseEvent me) {");
    	writer.write(NEWLINE);
    	if (options.getAllowBranchingOption()) {
	    	writer.write("if (me.getButton() != MouseEvent.BUTTON1) { // not left-click");
	    	writer.write(NEWLINE);
	    	writer.write("XYPlot plot = chart.getXYPlot();");
	    	writer.write(NEWLINE);
	    	writer.write("Range domainRange = plot.getDataRange(plot.getDomainAxis());");
	    	writer.write(NEWLINE);
	    	writer.write("if (domainRange != null) { // chart is not blank");
	    	writer.write(NEWLINE);
	    	writer.write("Point2D pt = chartPanel.translateScreenToJava2D(new Point(me.getX(), me.getY()));");
	    	writer.write(NEWLINE);
	    	writer.write("ChartRenderingInfo info = this.chartPanel.getChartRenderingInfo();");
	    	writer.write(NEWLINE);
	    	writer.write("Rectangle2D dataArea = info.getPlotInfo().getDataArea();");
	    	writer.write(NEWLINE);
	    	writer.write("NumberAxis domainAxis = (NumberAxis)plot.getDomainAxis();");
	    	writer.write(NEWLINE);
	    	writer.write("RectangleEdge domainAxisEdge = plot.getDomainAxisEdge();");
	    	writer.write(NEWLINE);
	    	writer.write("double chartX = domainAxis.java2DToValue(pt.getX(), dataArea, domainAxisEdge);");
	    	writer.write(NEWLINE);
	    	writer.write("lastRightClickedX = (int)Math.rint(chartX);");
	    	writer.write(NEWLINE);
	    	writer.write("if (domainRange != null && lastRightClickedX >= domainRange.getLowerBound() && lastRightClickedX <= (domainRange.getUpperBound() - 1) && lastRightClickedX >= 0) { // clicked within domain range");
	    	writer.write(NEWLINE);
	    	writer.write("if (chartPanel.getPopupMenu().getComponentIndex(newBranchItem) == -1) { // no new branch item on menu currently");
	    	writer.write(NEWLINE);
	    	writer.write("chartPanel.getPopupMenu().add(separator);");
	    	writer.write(NEWLINE);
	    	writer.write("chartPanel.getPopupMenu().add(newBranchItem);");
	    	writer.write(NEWLINE);
	    	writer.write("chartPanel.getPopupMenu().pack();");
	    	writer.write(NEWLINE);
	    	writer.write("chartPanel.getPopupMenu().repaint();");
	    	writer.write(NEWLINE);
	    	writer.write(CLOSED_BRACK);
	    	writer.write(NEWLINE);
	    	writer.write(CLOSED_BRACK);
	    	writer.write(NEWLINE);
	    	writer.write("else { // clicked outside of domain range");
	    	writer.write(NEWLINE);
	    	writer.write("if (chartPanel.getPopupMenu().getComponentIndex(newBranchItem) >= 0) { // new branch item currently on menu");
	    	writer.write(NEWLINE);
	    	writer.write("chartPanel.getPopupMenu().remove(newBranchItem);");
	    	writer.write(NEWLINE);
	    	writer.write("if (chartPanel.getPopupMenu().getComponentIndex(separator) >= 0) { // has separator");
	    	writer.write(NEWLINE);
	    	writer.write("chartPanel.getPopupMenu().remove(separator);");
	    	writer.write(NEWLINE);
	    	writer.write(CLOSED_BRACK);
	    	writer.write(NEWLINE);
	    	writer.write("chartPanel.getPopupMenu().pack();");
	    	writer.write(NEWLINE);
	    	writer.write("chartPanel.getPopupMenu().repaint();");
	    	writer.write(NEWLINE);
	    	writer.write(CLOSED_BRACK);
	    	writer.write(NEWLINE);
	    	writer.write(CLOSED_BRACK);
	    	writer.write(NEWLINE);
	    	writer.write(CLOSED_BRACK);
	    	writer.write(NEWLINE);
	    	writer.write(CLOSED_BRACK);
	    	writer.write(NEWLINE);
    	}
    	writer.write(CLOSED_BRACK);
    	writer.write(NEWLINE);
    	
    	// other mouseListener functions:
    	writer.write("public void mousePressed(MouseEvent me) {}");
    	writer.write(NEWLINE);
    	writer.write("public void mouseClicked(MouseEvent me) {}");
    	writer.write(NEWLINE);
    	writer.write("public void mouseEntered(MouseEvent me) {}");
    	writer.write(NEWLINE);
    	writer.write("public void mouseExited(MouseEvent me) {}");
    	writer.write(NEWLINE);
    	writer.write(NEWLINE);
    	
    	// actionPerformed function:
    	writer.write("public void actionPerformed(ActionEvent evt) {");
    	writer.write(NEWLINE);
    	writer.write("if (evt.getSource() == newBranchItem) {");
    	writer.write(NEWLINE);
			writer.write("String newBranchName = JOptionPane.showInputDialog(null, \"Please name this new game:\", \"Name New Game\", JOptionPane.QUESTION_MESSAGE);");
			writer.write(NEWLINE);
			writer.write("if (newBranchName != null) {");
			writer.write(NEWLINE);
    	writer.write("State tempState = (State)log.get(lastRightClickedX).clone();");
    	writer.write(NEWLINE);
    	writer.write("Logger tempLogger = new Logger(tempState, new ArrayList<State>(log.subList(0, lastRightClickedX)));");
    	writer.write(NEWLINE);
    	writer.write("Clock tempClock = new Clock(tempLogger, lastRightClickedX);");
    	writer.write(NEWLINE);
    	writer.write("tempState.setClock(tempClock);");
    	writer.write(NEWLINE);
    	writer.write("tempState.setLogger(tempLogger);");
    	writer.write(NEWLINE);
    	writer.write("SimSE.startNewBranch(tempState, new Branch(newBranchName, lastRightClickedX, tempClock.getTime(), branch, null));");
    	writer.write(NEWLINE);
    	writer.write(CLOSED_BRACK);
    	writer.write(NEWLINE);
    	writer.write(CLOSED_BRACK);
    	writer.write(NEWLINE);
    	writer.write(CLOSED_BRACK);
    	writer.write(NEWLINE);
    	writer.write(NEWLINE);      

      // "getXYPlot" method:
      writer.write("public XYPlot getXYPlot() {");
      writer.write(NEWLINE);
      writer.write("return chart.getXYPlot();");
      writer.write(NEWLINE);
      writer.write(CLOSED_BRACK);
      writer.write(NEWLINE);
      writer.write(NEWLINE);

      // "getChartTitle" method:
      writer.write("public String getChartTitle() {");
      writer.write(NEWLINE);
      writer.write("return this.getTitle();");
      writer.write(NEWLINE);
      writer.write(CLOSED_BRACK);
      writer.write(NEWLINE);
      writer.write(NEWLINE);
      
      // "getLog" method:
    	writer.write("public ArrayList<State> getLog() {");
    	writer.write(NEWLINE);
    	writer.write("return log;");
    	writer.write(NEWLINE);
    	writer.write(CLOSED_BRACK);
    	writer.write(NEWLINE);
    	writer.write(NEWLINE);

      // ExitListener class:
      writer.write("public class ExitListener extends WindowAdapter {");
      writer.write(NEWLINE);
      writer.write("public void windowClosing(WindowEvent event) {");
      writer.write(NEWLINE);
      writer.write("setVisible(false);");
      writer.write(NEWLINE);
      writer.write("dispose();");
      writer.write(NEWLINE);
      writer.write(CLOSED_BRACK);
      writer.write(NEWLINE);
      writer.write(CLOSED_BRACK);
      writer.write(NEWLINE);

      writer.write(CLOSED_BRACK);
      writer.close();
    } catch (IOException e) {
      JOptionPane.showMessageDialog(null, ("Error writing file "
          + objGraphFile.getPath() + ": " + e.toString()), "File IO Error",
          JOptionPane.WARNING_MESSAGE);
    }
  }
}