/*
 * This class is responsible for generating all of the code for the logic's
 * ChooseActionToDestroyJoinDialog component
 */

package simse.codegenerator.logicgenerator.dialoggenerator;

import simse.codegenerator.CodeGeneratorConstants;
import simse.codegenerator.CodeGeneratorUtils;

import simse.modelbuilder.actionbuilder.ActionType;
import simse.modelbuilder.actionbuilder.ActionTypeParticipant;
import simse.modelbuilder.actionbuilder.ActionTypeParticipantAttributeConstraint;
import simse.modelbuilder.actionbuilder.ActionTypeTrigger;
import simse.modelbuilder.actionbuilder.DefinedActionTypes;
import simse.modelbuilder.actionbuilder.AttributeGuard;
import simse.modelbuilder.actionbuilder.UserActionTypeTrigger;
import simse.modelbuilder.objectbuilder.AttributeTypes;
import simse.modelbuilder.objectbuilder.SimSEObjectType;
import simse.modelbuilder.objectbuilder.SimSEObjectTypeTypes;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import java.util.Vector;

import javax.swing.JOptionPane;

public class ChooseActionToJoinDialogGenerator implements
    CodeGeneratorConstants {
  private File directory; // directory to generate into
  private File catjdFile; // file to generate
  private DefinedActionTypes actTypes; // holds all of the defined action types

  public ChooseActionToJoinDialogGenerator(DefinedActionTypes actTypes, 
  		File directory) {
    this.directory = directory;
    this.actTypes = actTypes;
  }

  public void generate() {
    try {
      catjdFile = new File(directory,
          ("simse\\logic\\dialogs\\ChooseActionToJoinDialog.java"));
      if (catjdFile.exists()) {
        catjdFile.delete(); // delete old version of file
      }
      FileWriter writer = new FileWriter(catjdFile);
      writer
          .write("/* File generated by: simse.codegenerator.logicgenerator.dialoggenerator.ChooseActionToJoinDialogGenerator */");
      writer.write(NEWLINE);
      // package statement:
      writer.write("package simse.logic.dialogs;");
      writer.write(NEWLINE);
      // imports:
      writer.write("import simse.state.*;");
      writer.write(NEWLINE);
      writer.write("import simse.logic.*;");
      writer.write(NEWLINE);
      writer.write("import simse.adts.objects.*;");
      writer.write(NEWLINE);
      writer.write("import simse.adts.actions.*;");
      writer.write(NEWLINE);
      writer.write("import java.util.*;");
      writer.write(NEWLINE);
      writer.write("import javax.swing.*;");
      writer.write(NEWLINE);
      writer.write("import java.awt.event.*;");
      writer.write(NEWLINE);
      writer.write("import java.awt.*;");
      writer.write(NEWLINE);
      writer.write("import javax.swing.border.*;");
      writer.write(NEWLINE);
      writer.write("import javax.swing.event.*;");
      writer.write(NEWLINE);
      writer
          .write("public class ChooseActionToJoinDialog extends JDialog implements ActionListener");
      writer.write(NEWLINE);
      writer.write(OPEN_BRACK);
      writer.write(NEWLINE);
      
      // member variables:
      writer.write("private JFrame gui;");
      writer.write(NEWLINE);
      writer.write("private Vector<? extends simse.adts.actions.Action> actions;");
      writer.write(NEWLINE);
      writer.write("private State state;");
      writer.write(NEWLINE);
      writer.write("private Employee emp;");
      writer.write(NEWLINE);
      writer.write("private Vector<JRadioButton> radioButtons;");
      writer.write(NEWLINE);
      writer.write("private ButtonGroup radioButtonGroup;");
      writer.write(NEWLINE);
      writer.write("private JButton okButton;");
      writer.write(NEWLINE);
      writer.write("private JButton cancelButton;");
      writer.write(NEWLINE);
      writer.write("private String menuText;");
      writer.write(NEWLINE);
      writer.write("private RuleExecutor ruleExec;");
      writer.write(NEWLINE);

      // constructor:
      writer
          .write("public ChooseActionToJoinDialog(JFrame owner, Vector<? extends simse.adts.actions.Action> acts, Employee e, State s, String menText, RuleExecutor re)");
      writer.write(NEWLINE);
      writer.write(OPEN_BRACK);
      writer.write(NEWLINE);
      writer.write("super(owner, true);");
      writer.write(NEWLINE);
      writer.write("gui = owner;");
      writer.write(NEWLINE);
      writer.write("actions = acts;");
      writer.write(NEWLINE);
      writer.write("emp = e;");
      writer.write(NEWLINE);
      writer.write("state = s;");
      writer.write(NEWLINE);
      writer.write("menuText = menText;");
      writer.write(NEWLINE);
      writer.write("ruleExec = re;");
      writer.write(NEWLINE);
      writer.write("radioButtons = new Vector<JRadioButton>();");
      writer.write(NEWLINE);
      writer.write("radioButtonGroup = new ButtonGroup();");
      writer.write(NEWLINE);
      writer.write("setTitle(\"Join Action\");");
      writer.write(NEWLINE);
      // main pane:
      writer.write("Box mainPane = Box.createVerticalBox();");
      writer.write(NEWLINE);
      // top pane:
      writer.write("JPanel topPane = new JPanel();");
      writer.write(NEWLINE);
      writer.write("String actionName = new String();");
      writer.write(NEWLINE);
      writer
          .write("simse.adts.actions.Action tempAct = actions.elementAt(0);");
      writer.write(NEWLINE);

      // make a Vector of all the action types with user triggers:
      Vector<ActionType> userTrigActs = new Vector<ActionType>();
      Vector<ActionType> allActs = actTypes.getAllActionTypes();
      for (int j = 0; j < allActs.size(); j++) {
        ActionType userAct = allActs.elementAt(j);
        Vector<ActionTypeTrigger> allTrigs = userAct.getAllTriggers();
        for (int k = 0; k < allTrigs.size(); k++) {
          ActionTypeTrigger tempTrig = allTrigs.elementAt(k);
          if (tempTrig instanceof UserActionTypeTrigger) {
            userTrigActs.add(userAct);
            break;
          }
        }
      }
      // go through each action type and generate code for it:
      for (int j = 0; j < userTrigActs.size(); j++) {
        ActionType act = userTrigActs.elementAt(j);
        if (j > 0) { // not on first element
          writer.write("else ");
        }
        writer.write("if(tempAct instanceof "
            + CodeGeneratorUtils.getUpperCaseLeading(act.getName()) + "Action)");
        writer.write(NEWLINE);
        writer.write(OPEN_BRACK);
        writer.write(NEWLINE);
        writer.write("actionName = \"" + CodeGeneratorUtils.getUpperCaseLeading(
        		act.getName()) + "\";");
        writer.write(NEWLINE);
        writer.write(CLOSED_BRACK);
        writer.write(NEWLINE);
      }
      writer
          .write("topPane.add(new JLabel(\"Choose which \" + actionName + \" Action to join:\"));");
      writer.write(NEWLINE);
      writer.write("JPanel middlePane = new JPanel(new GridLayout(0, 1));");
      writer.write(NEWLINE);
      for (int j = 0; j < userTrigActs.size(); j++) {
        ActionType act = userTrigActs.elementAt(j);
        if (j > 0) { // not on first element
          writer.write("else ");
        }
        writer.write("if(tempAct instanceof "
            + CodeGeneratorUtils.getUpperCaseLeading(act.getName()) + 
            "Action)");
        writer.write(NEWLINE);
        writer.write(OPEN_BRACK);
        writer.write(NEWLINE);
        writer.write("for(int i=0; i<actions.size(); i++)");
        writer.write(NEWLINE);
        writer.write(OPEN_BRACK);
        writer.write(NEWLINE);
        writer.write(CodeGeneratorUtils.getUpperCaseLeading(act.getName()) + 
        		"Action act = ("
            + CodeGeneratorUtils.getUpperCaseLeading(act.getName())
            + "Action)actions.elementAt(i);");
        writer.write(NEWLINE);
        writer.write("StringBuffer label = new StringBuffer(\"<html>\");");
        writer.write(NEWLINE);
        // go through all participants:
        Vector<ActionTypeParticipant> parts = act.getAllParticipants();
        for (int k = 0; k < parts.size(); k++) {
          if (k > 0) { // not on first element
            writer.write("label.append(\"; <br>\");");
            writer.write(NEWLINE);
          }
          ActionTypeParticipant tempPart = parts.elementAt(k);
          String metaTypeName = CodeGeneratorUtils.getUpperCaseLeading(
          		SimSEObjectTypeTypes.getText(tempPart.getSimSEObjectTypeType()));
          writer.write("label.append(\"" + tempPart.getName() + "(s): \");");
          writer.write(NEWLINE);
          writer.write("Vector<" + metaTypeName + "> all" + tempPart.getName() +
          		"s = act.getAll" + tempPart.getName() + "s();");
          writer.write(NEWLINE);
          writer.write("for(int j=0; j<all" + tempPart.getName()
              + "s.size(); j++)");
          writer.write(NEWLINE);
          writer.write(OPEN_BRACK);
          writer.write(NEWLINE);
          writer.write("if(j > 0)");
          writer.write(NEWLINE);
          writer.write(OPEN_BRACK);
          writer.write(NEWLINE);
          writer.write("label.append(\", \");");
          writer.write(NEWLINE);
          writer.write(CLOSED_BRACK);
          writer.write(NEWLINE);
          writer.write(SimSEObjectTypeTypes.getText(tempPart
              .getSimSEObjectTypeType())
              + " a = all" + tempPart.getName() + "s.elementAt(j);");
          writer.write(NEWLINE);
          // go through all allowable SimSEObjectTypes for this participant:
          Vector<SimSEObjectType> ssObjTypes = tempPart.getAllSimSEObjectTypes();
          for (int m = 0; m < ssObjTypes.size(); m++) {
            SimSEObjectType tempType = ssObjTypes.elementAt(m);
            if (m > 0) { // not on first element
              writer.write("else ");
            }
            writer.write("if(a instanceof "
                + CodeGeneratorUtils.getUpperCaseLeading(tempType.getName()) + 
                ")");
            writer.write(NEWLINE);
            writer.write(OPEN_BRACK);
            writer.write(NEWLINE);
            writer.write("label.append(\"" + tempType.getName() + "(\" + (("
                + CodeGeneratorUtils.getUpperCaseLeading(tempType.getName()) + 
                ")a).get" + 
                CodeGeneratorUtils.getUpperCaseLeading(
                		tempType.getKey().getName()) + "() + \")\");");
            writer.write(NEWLINE);
            writer.write(CLOSED_BRACK);
            writer.write(NEWLINE);
          }
          writer.write(CLOSED_BRACK);
          writer.write(NEWLINE);
        }
        writer.write("label.append(\"</HTML>\");");
        writer.write(NEWLINE);
        writer.write("JPanel tempPane = new JPanel(new BorderLayout());");
        writer.write(NEWLINE);
        writer
            .write("JRadioButton tempRadioButton = new JRadioButton(label.toString());");
        writer.write(NEWLINE);
        writer.write("radioButtonGroup.add(tempRadioButton);");
        writer.write(NEWLINE);
        writer.write("tempPane.add(tempRadioButton, BorderLayout.WEST);");
        writer.write(NEWLINE);
        writer.write("radioButtons.add(tempRadioButton);");
        writer.write(NEWLINE);
        writer.write("middlePane.add(tempPane);");
        writer.write(NEWLINE);
        writer.write(CLOSED_BRACK);
        writer.write(NEWLINE);
        writer.write(CLOSED_BRACK);
        writer.write(NEWLINE);
      }

      // bottom pane:
      writer.write("JPanel bottomPane = new JPanel();");
      writer.write(NEWLINE);
      writer.write("okButton = new JButton(\"OK\");");
      writer.write(NEWLINE);
      writer.write("okButton.addActionListener(this);");
      writer.write(NEWLINE);
      writer.write("bottomPane.add(okButton);");
      writer.write(NEWLINE);
      writer.write("cancelButton = new JButton(\"Cancel\");");
      writer.write(NEWLINE);
      writer.write("cancelButton.addActionListener(this);");
      writer.write(NEWLINE);
      writer.write("bottomPane.add(cancelButton);");
      writer.write(NEWLINE);

      // add panes to main pane:
      writer.write("mainPane.add(topPane);");
      writer.write(NEWLINE);
      writer.write("mainPane.add(middlePane);");
      writer.write(NEWLINE);
      writer.write("mainPane.add(bottomPane);");
      writer.write(NEWLINE);

      // Set main window frame properties:
      writer.write("setContentPane(mainPane);");
      writer.write(NEWLINE);
      writer.write("validate();");
      writer.write(NEWLINE);
      writer.write("pack();");
      writer.write(NEWLINE);
      writer.write("repaint();");
      writer.write(NEWLINE);
      writer.write("toFront();");
      writer.write(NEWLINE);
      writer.write("Point ownerLoc = owner.getLocationOnScreen();");
      writer.write(NEWLINE);
      writer.write("Point thisLoc = new Point();");
      writer.write(NEWLINE);
      writer
          .write("thisLoc.setLocation((ownerLoc.getX() + (owner.getWidth() / 2) - (this.getWidth() / 2)), (ownerLoc.getY() + (owner.getHeight() / 2) - (this.getHeight() / 2)));");
      writer.write(NEWLINE);
      writer.write("setLocation(thisLoc);");
      writer.write(NEWLINE);
      writer.write("if(radioButtons.size() == 1)");
      writer.write(NEWLINE);
      writer.write(OPEN_BRACK);
      writer.write(NEWLINE);
      writer.write("onlyOneChoice(owner);");
      writer.write(NEWLINE);
      writer.write(CLOSED_BRACK);
      writer.write(NEWLINE);
      writer.write("else");
      writer.write(NEWLINE);
      writer.write(OPEN_BRACK);
      writer.write(NEWLINE);
      writer.write("setVisible(true);");
      writer.write(NEWLINE);
      writer.write(CLOSED_BRACK);
      writer.write(NEWLINE);
      writer.write(CLOSED_BRACK);
      writer.write(NEWLINE);

      // actionPerformed function:
      writer.write("public void actionPerformed(ActionEvent evt)");
      writer.write(NEWLINE);
      writer.write(OPEN_BRACK);
      writer.write(NEWLINE);
      writer.write("Object source = evt.getSource();");
      writer.write(NEWLINE);
      writer.write("if(source == cancelButton)");
      writer.write(NEWLINE);
      writer.write(OPEN_BRACK);
      writer.write(NEWLINE);
      writer.write("setVisible(false);");
      writer.write(NEWLINE);
      writer.write("dispose();");
      writer.write(NEWLINE);
      writer.write(CLOSED_BRACK);
      writer.write(NEWLINE);
      writer.write("else if(source == okButton)");
      writer.write(NEWLINE);
      writer.write(OPEN_BRACK);
      writer.write(NEWLINE);
      writer.write("boolean anySelected = false;");
      writer.write(NEWLINE);
      writer.write("for(int i=0; i<radioButtons.size(); i++)");
      writer.write(NEWLINE);
      writer.write(OPEN_BRACK);
      writer.write(NEWLINE);
      writer
          .write("JRadioButton tempRButt = radioButtons.elementAt(i);");
      writer.write(NEWLINE);
      writer.write("if(tempRButt.isSelected())");
      writer.write(NEWLINE);
      writer.write(OPEN_BRACK);
      writer.write(NEWLINE);
      writer.write("anySelected = true;");
      writer.write(NEWLINE);
      writer.write("break;");
      writer.write(NEWLINE);
      writer.write(CLOSED_BRACK);
      writer.write(NEWLINE);
      writer.write(CLOSED_BRACK);
      writer.write(NEWLINE);
      writer.write("if(!anySelected)");
      writer.write(NEWLINE);
      writer.write(OPEN_BRACK);
      writer.write(NEWLINE);
      writer
          .write("JOptionPane.showMessageDialog(null, (\"You must choose at least one action\"), \"Invalid Input\", JOptionPane.ERROR_MESSAGE);");
      writer.write(NEWLINE);
      writer.write(CLOSED_BRACK);
      writer.write(NEWLINE);
      writer.write("else");
      writer.write(NEWLINE);
      writer.write(OPEN_BRACK);
      writer.write(NEWLINE);
      writer.write("for(int i=0; i<radioButtons.size(); i++)");
      writer.write(NEWLINE);
      writer.write(OPEN_BRACK);
      writer.write(NEWLINE);
      writer
          .write("JRadioButton rButt  = radioButtons.elementAt(i);");
      writer.write(NEWLINE);
      writer.write("if(rButt.isSelected())");
      writer.write(NEWLINE);
      writer.write(OPEN_BRACK);
      writer.write(NEWLINE);
      writer
          .write("simse.adts.actions.Action tempAct = actions.elementAt(i);");
      writer.write(NEWLINE);
      writer.write("Vector<String> participantNames = new Vector<String>();");
      writer.write(NEWLINE);

      // go through all action types w/ user triggers:
      for (int i = 0; i < userTrigActs.size(); i++) {
        ActionType tempAct = userTrigActs.elementAt(i);
        if (i > 0) { // not on first element
          writer.write("else ");
        }
        writer.write("if(tempAct instanceof "
            + CodeGeneratorUtils.getUpperCaseLeading(tempAct.getName()) + 
            "Action)");
        writer.write(NEWLINE);
        writer.write(OPEN_BRACK);
        writer.write(NEWLINE);

        // go through all employee participants:
        Vector<ActionTypeParticipant> allParts = tempAct.getAllParticipants();
        for (int j = 0; j < allParts.size(); j++) {
          ActionTypeParticipant part = allParts.elementAt(j);
          if (part.getSimSEObjectTypeType() == SimSEObjectTypeTypes.EMPLOYEE) { 
          	// employee participant
            writer.write("Vector all" + part.getName() + "s = (("
                + CodeGeneratorUtils.getUpperCaseLeading(tempAct.getName())
                + "Action)tempAct).getAll" + part.getName() + "s();");
            writer.write(NEWLINE);
            writer.write("if((all" + part.getName()
                + "s.contains(emp) == false) && (all" + part.getName()
                + "s.size() < ");
            if (part.getQuantity().isMaxValBoundless()) {
              writer.write("999999");
            } else { // max val has a value
              writer.write(part.getQuantity().getMaxVal().toString());
            }
            writer.write("))");
            writer.write(NEWLINE);
            writer.write(OPEN_BRACK);
            writer.write(NEWLINE);

            // collect all the user triggers for this action:
            Vector<UserActionTypeTrigger> userTrigsTemp = 
            	new Vector<UserActionTypeTrigger>();
            Vector<ActionTypeTrigger> allTriggers = tempAct.getAllTriggers();
            for (int k = 0; k < allTriggers.size(); k++) {
              ActionTypeTrigger tempTrig = allTriggers.elementAt(k);
              if (tempTrig instanceof UserActionTypeTrigger) {
                userTrigsTemp.add((UserActionTypeTrigger)tempTrig);
              }
            }

            // go through all of the user triggers:
            for (int k = 0; k < userTrigsTemp.size(); k++) {
              UserActionTypeTrigger userTrig = userTrigsTemp.elementAt(k);
              if (k > 0) { // not on first element
                writer.write("else ");
              }
              writer.write("if(menuText.equals(\"" + userTrig.getMenuText()
                  + "\"))");
              writer.write(NEWLINE);
              writer.write(OPEN_BRACK);
              writer.write(NEWLINE);

              // go through all allowable types:
              Vector<SimSEObjectType> types = part.getAllSimSEObjectTypes();
              for (int m = 0; m < types.size(); m++) {
                SimSEObjectType type = types.elementAt(m);
                if (m > 0) { // not on first element
                  writer.write("else ");
                }
                writer.write("if((emp instanceof "
                    + CodeGeneratorUtils.getUpperCaseLeading(type.getName()) + 
                    ")");

                // go through all attribute constraints:
                ActionTypeParticipantAttributeConstraint[] attConstraints = 
                	userTrig.getParticipantTrigger(part.getName()).getConstraint(
                        type.getName()).getAllAttributeConstraints();
                for (int n = 0; n < attConstraints.length; n++) {
                  ActionTypeParticipantAttributeConstraint attConst = 
                  	attConstraints[n];
                  if (attConst.isConstrained()) {
                    writer
                        .write(" && ((("
                            + CodeGeneratorUtils.getUpperCaseLeading(
                            		type.getName()) + ")emp).get" + 
                            		CodeGeneratorUtils.getUpperCaseLeading(
                            				attConst.getAttribute().getName()) + "() ");
                    if (attConst.getAttribute().getType() == 
                    	AttributeTypes.STRING) {
                      writer.write(".equals(" + "\""
                          + attConst.getValue().toString() + "\")");
                    } else {
                      if (attConst.getGuard().equals(AttributeGuard.EQUALS)) {
                        writer.write("== ");
                      } else {
                        writer.write(attConst.getGuard() + " ");
                      }
                      writer.write(attConst.getValue().toString());
                    }
                    writer.write(")");
                  }
                }
                writer.write(")");
                writer.write(NEWLINE);
                writer.write(OPEN_BRACK);
                writer.write(NEWLINE);
                writer.write("participantNames.add(\""
                    + CodeGeneratorUtils.getUpperCaseLeading(part.getName()) + 
                    "\");");
                writer.write(NEWLINE);
                writer.write(CLOSED_BRACK);
                writer.write(NEWLINE);
              }
              writer.write(CLOSED_BRACK);
              writer.write(NEWLINE);
            }
            writer.write(CLOSED_BRACK);
            writer.write(NEWLINE);
          }
        }
        writer.write(CLOSED_BRACK);
        writer.write(NEWLINE);
      }
      writer
          .write("new ChooseRoleToPlayDialog(gui, participantNames, emp, tempAct, menuText, ruleExec);");
      writer.write(NEWLINE);
      writer.write("setVisible(false);");
      writer.write(NEWLINE);
      writer.write("dispose();");
      writer.write(NEWLINE);
      writer.write("break;");
      writer.write(NEWLINE);
      writer.write(CLOSED_BRACK);
      writer.write(NEWLINE);
      writer.write(CLOSED_BRACK);
      writer.write(NEWLINE);
      writer.write(CLOSED_BRACK);
      writer.write(NEWLINE);
      writer.write(CLOSED_BRACK);
      writer.write(NEWLINE);
      writer.write(CLOSED_BRACK);
      writer.write(NEWLINE);

      // onlyOneChoice function:
      writer.write("private void onlyOneChoice(JFrame owner)");
      writer.write(NEWLINE);
      writer.write(OPEN_BRACK);
      writer.write(NEWLINE);
      writer.write("for(int i=0; i<radioButtons.size(); i++)");
      writer.write(NEWLINE);
      writer.write(OPEN_BRACK);
      writer.write(NEWLINE);
      writer
          .write("JRadioButton rButt = radioButtons.elementAt(i);");
      writer.write(NEWLINE);
      writer
          .write("simse.adts.actions.Action tempAct = actions.elementAt(i);");
      writer.write(NEWLINE);
      writer.write("Vector<String> participantNames = new Vector<String>();");
      writer.write(NEWLINE);

      // go through all action types w/ user triggers:
      for (int i = 0; i < userTrigActs.size(); i++) {
        ActionType tempAct = userTrigActs.elementAt(i);
        if (i > 0) { // not on first element
          writer.write("else ");
        }
        writer.write("if(tempAct instanceof "
            + CodeGeneratorUtils.getUpperCaseLeading(tempAct.getName()) + 
            "Action)");
        writer.write(NEWLINE);
        writer.write(OPEN_BRACK);
        writer.write(NEWLINE);

        // go through all employee participants:
        Vector<ActionTypeParticipant> allParts = tempAct.getAllParticipants();
        for (int j = 0; j < allParts.size(); j++) {
          ActionTypeParticipant part = allParts.elementAt(j);
          if (part.getSimSEObjectTypeType() == SimSEObjectTypeTypes.EMPLOYEE) { 
          	// employee participant
            writer.write("Vector all" + part.getName() + "s = (("
                + CodeGeneratorUtils.getUpperCaseLeading(tempAct.getName())
                + "Action)tempAct).getAll" + part.getName() + "s();");
            writer.write(NEWLINE);
            writer.write("if((all" + part.getName()
                + "s.contains(emp) == false) && (all" + part.getName()
                + "s.size() < ");
            if (part.getQuantity().isMaxValBoundless()) {
              writer.write("999999");
            } else { // max val has a value
              writer.write(part.getQuantity().getMaxVal().toString());
            }
            writer.write("))");
            writer.write(NEWLINE);
            writer.write(OPEN_BRACK);
            writer.write(NEWLINE);

            // collect all the user triggers for this action:
            Vector<UserActionTypeTrigger> userTrigsTemp = 
            	new Vector<UserActionTypeTrigger>();
            Vector<ActionTypeTrigger> allTriggers = tempAct.getAllTriggers();
            for (int k = 0; k < allTriggers.size(); k++) {
              ActionTypeTrigger tempTrig = allTriggers.elementAt(k);
              if (tempTrig instanceof UserActionTypeTrigger) {
                userTrigsTemp.add((UserActionTypeTrigger)tempTrig);
              }
            }

            // go through all of the user triggers:
            for (int k = 0; k < userTrigsTemp.size(); k++) {
              UserActionTypeTrigger userTrig = userTrigsTemp.elementAt(k);
              if (k > 0) { // not on first element
                writer.write("else ");
              }
              writer.write("if(menuText.equals(\"" + userTrig.getMenuText()
                  + "\"))");
              writer.write(NEWLINE);
              writer.write(OPEN_BRACK);
              writer.write(NEWLINE);

              // go through all allowable types:
              Vector<SimSEObjectType> types = part.getAllSimSEObjectTypes();
              for (int m = 0; m < types.size(); m++) {
                SimSEObjectType type = types.elementAt(m);
                if (m > 0) { // not on first element
                  writer.write("else ");
                }
                writer.write("if((emp instanceof "
                    + CodeGeneratorUtils.getUpperCaseLeading(type.getName()) + 
                    ")");

                // go through all attribute constraints:
                ActionTypeParticipantAttributeConstraint[] attConstraints = 
                	userTrig.getParticipantTrigger(part.getName()).getConstraint(
                        type.getName()).getAllAttributeConstraints();
                for (int n = 0; n < attConstraints.length; n++) {
                  ActionTypeParticipantAttributeConstraint attConst = 
                  	attConstraints[n];
                  if (attConst.isConstrained()) {
                    writer
                        .write(" && ((("
                            + CodeGeneratorUtils.getUpperCaseLeading(
                            		type.getName()) + ")emp).get" + 
                            		CodeGeneratorUtils.getUpperCaseLeading(
                            				attConst.getAttribute().getName()) + "() ");
                    if (attConst.getAttribute().getType() == 
                    	AttributeTypes.STRING) {
                      writer.write(".equals(" + "\""
                          + attConst.getValue().toString() + "\")");
                    } else {
                      if (attConst.getGuard().equals(AttributeGuard.EQUALS)) {
                        writer.write("== ");
                      } else {
                        writer.write(attConst.getGuard() + " ");
                      }
                      writer.write(attConst.getValue().toString());
                    }
                    writer.write(")");
                  }
                }
                writer.write(")");
                writer.write(NEWLINE);
                writer.write(OPEN_BRACK);
                writer.write(NEWLINE);
                writer.write("participantNames.add(\""
                    + CodeGeneratorUtils.getUpperCaseLeading(part.getName()) + 
                    "\");");
                writer.write(NEWLINE);
                writer.write(CLOSED_BRACK);
                writer.write(NEWLINE);
              }
              writer.write(CLOSED_BRACK);
              writer.write(NEWLINE);
            }
            writer.write(CLOSED_BRACK);
            writer.write(NEWLINE);
          }
        }
        writer.write(CLOSED_BRACK);
        writer.write(NEWLINE);
      }
      writer
          .write("new ChooseRoleToPlayDialog(owner, participantNames, emp, tempAct, menuText, ruleExec);");
      writer.write(NEWLINE);
      writer.write("dispose();");
      writer.write(NEWLINE);
      writer.write("break;");
      writer.write(NEWLINE);
      writer.write(CLOSED_BRACK);
      writer.write(NEWLINE);
      writer.write(CLOSED_BRACK);
      writer.write(NEWLINE);
      writer.write(CLOSED_BRACK);
      writer.write(NEWLINE);

      writer.close();
    } catch (IOException e) {
      JOptionPane.showMessageDialog(null, ("Error writing file "
          + catjdFile.getPath() + ": " + e.toString()), "File IO Error",
          JOptionPane.WARNING_MESSAGE);
    }
  }
}